<?php

declare(strict_types=1);

namespace Drupal\simple_push_local_web_push\Plugin\SimplePushBackend;

use Drupal\web_push\Service\WebPushSender;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Psr\Log\LoggerInterface;
use Drupal\simple_push\Attribute\SimplePushBackend;
use Drupal\simple_push\SimplePushBackendPluginBase;
use Drupal\web_push\Service\WebPushManager;

/**
 * Plugin implementation of the simple_push_backend.
 */
#[SimplePushBackend(
  id: 'local_web_push',
  label: new TranslatableMarkup('Local Web Push'),
  description: new TranslatableMarkup('Local web push backend.'),
  settings: 'simple_push.backend.local_web_push.settings'
)]
final class LocalWebPush extends SimplePushBackendPluginBase {
  /**
   * The WebPush Service.
   *
   * @var \Drupal\web_push\Service\WebPushManager
   */
  protected $webPushManager;

  /**
   * {@inheritdoc}
   * Construct a LocalWebPush simple_push plugin object.
   *
   * @param \Drupal\web_push\Service\WebPushManager $web_push_manager
   *   The WebPushManager Service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The responsible for providing language support on language-unaware sites.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, EntityFieldManagerInterface $entity_field_manager, ConfigFactoryInterface $config_factory, LoggerInterface $logger, LanguageManagerInterface $language_manager, WebPushManager $web_push_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $renderer, $entity_field_manager, $config_factory, $logger, $language_manager);
    $this->setConfiguration($configuration);
    $this->webPushManager = $web_push_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('entity_field.manager'),
      $container->get('config.factory'),
      $container->get('logger.channel.simple_push'),
      $container->get('language_manager'),
      $container->get('web_push.manager')
    );
  }

  /**
   *
   */
  public function addNotification(array $data):array {
    if (isset($data['entity'])) {
      $entity = $data['entity'];
      $site_config = $this->configFactory->get('system.site');
      $metatag = metatag_generate_entity_all_tags($entity);
      // $entity->title->view('teaser')
      $title = $metatag['title']['#attributes']['content'] ?? $entity->title->value;
      $contents = $metatag['description']['#attributes']['content'] ?? (string) $this->t("Read more on %name %slogan", [
        "%name" => $site_config->get('name'),
        "%slogan" => $site_config->get('slogan'),
      ]);
      $base_url = Url::fromUserInput('/', ['absolute' => TRUE])->toString();
      $picture = $metatag['og_image_0']['#attributes']['content'] ?? $base_url . theme_get_setting('logo.url');
      $url = $entity->toUrl()->setAbsolute()->toString();

      $this->webPushManager->sendNotification(
        $title,
        $contents,
        $url,
        $picture,
        WebPushSender::URGENCY_HIGH,
        "Node{$entity->id()}",
      // @todo get subscriptions ids [ 1, 2 ]
        NULL
      );
    }
    return [];
  }

}
