<?php

declare(strict_types=1);

namespace Drupal\simple_push\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;

/**
 * Returns responses for simple push routes.
 */
final class SimplePushBackendListBuilder extends ControllerBase {

  /**
   * Builds the plugin list form.
   */
  public function buildPluginList() {
    $build = [];

    $build['plugin_table'] = [
      '#type' => 'table',
      '#header' => $this->buildHeader(),
      '#rows' => [],
      // '#attributes' => ['id' => 'plugin-table-wrapper'],
      '#empty' => $this->t('No backends found'),
    ];

    foreach ($this->getPlugins() as $plugin) {
      $build['plugin_table']['#rows'][] = [
        'id' => $plugin['id'],
        'provider' => $plugin['provider'],
        'status' => $plugin['status'],
        'edit' => Link::createFromRoute($this->t('edit'), 'simple_push.simple_push_backend', ['id' => $plugin['id']]),
      ];
    }
    return $build;
  }

  /**
   * Gets the list of plugins.
   */
  protected function getPlugins() {
    // Replace 'my_custom_plugin' with your actual plugin type.
    $plugin_manager = \Drupal::service('plugin.manager.simple_push_backend');
    $plugins = $plugin_manager->getDefinitions();
    // $config = \Drupal::config('simple_push.settings');
    $plugin_list = [];
    foreach ($plugins as $plugin_id => $plugin) {
      $config = \Drupal::config($plugin['settings'])->get();
      $plugin_instance = $plugin_manager->createInstance($plugin_id, $config);
      $plugin_list[$plugin_id] = [
        'id' => $plugin_id,
        'provider' => $plugin['provider'],
        'status' => $plugin_instance->getConfiguration()['status'] ? $this->t("Enabled") : $this->t("Disabled"),
      ];
    }

    return $plugin_list;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];
    $header['id'] = $this->t('Backend');
    $header['provider'] = $this->t('Provider');
    $header['status'] = $this->t('Status');
    $header['edit'] = $this->t('Edit');
    return $header;
  }

}

/*
 *
 *
 * Schema to be deleted TODO!
simple_push.simple_push_backend.*:
type: config_entity
label: simple push backend
mapping:
id:
type: string
label: ID
label:
type: label
label: Label
uuid:
type: string

 */
