<?php

declare(strict_types=1);

namespace Drupal\simple_push\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_push\SimplePushBackendPluginManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Configure simple push settings for this site.
 */
final class SimplePushBackendForm extends ConfigFormBase {

  protected $pluginManager;
  protected $pluginId;
  protected $configName;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_push_simple_push_backend';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [$this->configName];
  }

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(ConfigFactoryInterface $config_factory, SimplePushBackendPluginManager $plugin_manager) {
    parent::__construct($config_factory);
    $this->pluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('config.factory'), $container->get('plugin.manager.simple_push_backend'));
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?string $id = NULL): array {
    $this->configName = $this->pluginManager->getDefinitions()[$id]['settings'];
    $this->pluginId = $id;
    $config = $this->configFactory()->get($this->configName)->get();
    $plugin_instance = $this->pluginManager->createInstance($id, $config);
    $plugin_instance->getConfiguration();
    $form['plugin__' . $id] = $plugin_instance->buildConfigurationForm($form, $form_state);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $instance = $this->pluginManager->createInstance($this->pluginId);
    // etConfiguration([])
    $instance->validateConfigurationForm($form, $form_state);

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $instance = $this->pluginManager->createInstance($this->pluginId);
    $instance->submitConfigurationForm($form, $form_state);
    $this->configFactory->getEditable($this->configName)
      ->setData($instance->getConfiguration())
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account, string $id) {
    // Check permissions and combine that with any custom access checking needed. Pass forward
    // parameters from the route and/or request as needed.
    return AccessResult::allowedIf($account->hasPermission('administer simple_push_backend') && isset($this->pluginManager->getDefinitions()[$id]));
  }

}
