<?php

namespace Drupal\simple_push\Hook;

use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 *
 */
class NodeHooks {
  use StringTranslationTrait;
  /**
   * The OneSignal service (todo move to plugin).
   *
   * @var \Drupal\onesignal\OneSignalService
   */
  protected $oneSignalService;

  /**
   * NodeHooks constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler) {
    $this->oneSignalService = \Drupal::service('onesignal');
  }

  /**
 *
 */
  #[Hook('node_insert')]
  #[Hook('node_update')]
  public function nodeUpdate(ContentEntityInterface $entity) {
    // Node update processing.
    // We need some condition before doing for every node.
    $type = FALSE;
    if ($this->isPublished($entity) && $type) {
      $push_manager = \Drupal::service('plugin.manager.simple_push_backend');
      $plugin_definitions = $push_manager->getDefinitions();
      foreach ($plugin_definitions as $plugin) {
        $config = \Drupal::config($plugin['settings'])->get();
        if ($config['status']) {
          $plugin = $push_manager->createInstance($plugin['id'], $config);
          $plugin->addNotification(['entity' => $entity]);
        }
      }
      // dump($plugin_definitions);
    }
  }

  /**
   * Determines if the given entity is published.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity.
   *
   * @return bool
   *   TRUE, if the content entity is published, FALSE otherwise.
   */
  protected function isPublished(ContentEntityInterface $entity): bool {
    if ($entity->isNew()) {
      return FALSE;
    }
    $entityType = $entity->getEntityType();
    if ($entityType->hasKey('status')) {
      $key = $entityType->getKey('status');
    }
    elseif ($entityType->hasKey('published')) {
      $key = $entityType->getKey('published');
    }
    else {
      return TRUE;
    }
    return (bool) $entity->get($key)->value;
  }

}
